package Maypole::Constants;
use base 'Exporter';
use constant OK       => 0;
use constant DECLINED => -1;
use constant ERROR    => -1;
our @EXPORT = qw(OK DECLINED ERROR);

=head1 NAME

Maypole::Constants - Maypole predefined constants

=head1 DESCRIPTION

This class defines constants for use with Maypole

=head1 SEE ALSO

L<Maypole>

=head1 MAINTAINER

Sebastian Riedel, c<sri@oook.de>

=head1 AUTHOR

Simon Cozens, C<simon@cpan.org>

=head1 LICENSE

You may distribute this code under the same terms as Perl itself.

=cut
1;
